%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function estimates the group memberships and parameters (slope 
% coefficients and threshold) in the panel structural threshold 
% regression model with common threshold value
%
% Input: 
% idxdata = data
% G = the number of groups
% h = heteroscedasticity indicator
% 
% Output:
% [theta1_best theta2_best] = estimated slope coefficients
% [se1_best se2_best] = estimated standard deviations
% gamma_best = estimated threshold
% gi_best = estimated group membership
% resQ_best = sum of squared residuals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [theta1_best, theta2_best, gamma_best, gammacf1, gammacf2, gi_best, se1_best, se2_best, resQ_best] = est_pstr_comthd(idxdata,G,h,confi)

N               = idxdata.N(end); 
T               = size(idxdata,1)/N;
k               = size(idxdata.X,2);
Y               = idxdata.y;
X               = idxdata.X;
q               = idxdata.q;
n               = length(X);
%% overall estimation
mi              = inv(X'*X);
beta            = mi*(X'*Y);
e               = Y-X*beta;
ee              = e'*e;
sig             = ee/(n-k);
xe              = X.*(e*ones(1,length(X(1,:))));

%% threshold estimation
Nsim            = 1000;                                                     % number of simulations
resQ_best       = 10^7;                                                     % sum of squared residuals for the best initial values
max_iteration   = 20;
qs              = quantile(q,[0.025:0.01:0.975]);
qn              = length(qs);
% qs              = sort(unique(q));
censor          = 0.05;                                                     % trim extreme threshold
qn_start        = round(censor*length(qs));
qn_end          = round((1-censor)*length(qs));
qs              = qs(qn_start:qn_end);
qn              = length(qs);
Q_best          = realmax;  
sn              = zeros(qn,1);
for rr = 1:qn
    gamma       = qs(rr);
    [est_theta1, est_theta2, est_gi, ~, res] = est_pstr_known_threshold(idxdata,G,gamma,h); % group estimation for each given gamma
    sn(rr)      = res'*res;
    Q           = sn(rr);
     if Q < Q_best
         Q_best    = Q;
         hattheta1 = est_theta1;
         hattheta2 = est_theta2;
         gi        = est_gi;
         qhat      = gamma;
     end
end
smin            = Q_best;
sighat          = smin/n;
gamma_best      = qhat;

%% coefficient estimation 
[theta1_best, theta2_best, gi_best, resQ_best, ~, se1_best, se2_best] = est_pstr_known_threshold(idxdata,G,qhat,h); % group estimation for each given gamma

%% confidence interval of threshold 
for g = 1:G                
    NN                      = 1:N;    
    gi                      = logical(gi_best);    
    this_group              = gi(:,g);        
    g_index                 = NN(this_group);        
    g_data                  = idxdata(ismember(idxdata.N,g_index),:);        
    Ng                      = sum(gi(:,g));                             
    gX                      = g_data.X;    
    gy                      = g_data.y;    
    gq                      = g_data.q;                            
    i1                      = (gq<=gamma); 
    i2                      = ~(i1);         
    x1                      = gX(i1,:);        
    y1                      = gy(i1,:);        
    x2                      = gX(i2,:);        
    y2                      = gy(i2,:);
    mi1                     = inv(x1'*x1);
    mi2                     = inv(x2'*x2);
    beta1                   = mi1*(x1'*y1);
    beta2                   = mi2*(x2'*y2);     
    e1                      = y1-x1*beta1;
    e2                      = y2-x2*beta2;
    eful                    = [e1;e2]; 
    
    nonpar_                 = 2;    
    r1                      = (gX*(beta1-beta2)).*(gX*(beta1-beta2));    
    r2                      = r1.*((eful.*eful)*ones(1,length(r1(1,:))));
    gqx                     = [gq.^0,gq.^1,gq.^2];    
    gqh                     = [qhat.^0,qhat.^1,qhat.^2];    
    m1                      = (r1'/gqx')';    
    m2                      = (r2'/gqx')';    
    g1                      = gqh*m1;    
    g2                      = gqh*m2;
    if nonpar_== 2
        sigq    = sqrt(mean((gq-mean(gq)').^2));
        hband   = 2.34*sigq/(Ng^(.25));
        u       = (qhat-gq)/hband;
        u2      = u.^2;
        f       = mean((1-u2).*(u2<=1))'*(1.5/(hband^2));
        df      = -mean((1-u2).*(u2<=1))'*(1.5/(hband^2));
        eps     = r1-gqx*m1;
        sige    = (eps'*eps)/(Ng-3);
        hband   = sige/(4*f*((m1(3)+(m1(2)+2*m1(3)*qhat)*df/f)^2));
        u2      = ((qhat-gq)/hband).^2;
        kh      = ((1-u2)*.75/hband).*(u2<=1);
        gg1(g)  = mean(kh.*(r1*ones(1,length(kh(1,:)))))';
        gg2(g)  = mean(kh.*(r2*ones(1,length(kh(1,:)))))';
    end
clear e1 e2 eful x1 x2 y1 y2 i1 i2 r1 r2
end
eta2            = sum(gg2)/sum(gg1);
lr              = (sn-smin)/eta2;
c1              = -2*log(1-sqrt(confi));
lr1             = (lr>=c1);
if max(lr1)==1
    minind_lr1  = 0;
    for i = 1:length(lr1)
        if lr1(i)==min(lr1)
            minind_lr1 = i;
            break;
        end
    end
    qhat1       = qs(minind_lr1);
    minind_lr1v = 0;
    for i = 1:length(lr1)
        if lr1(qn+1-i)==min(lr1)
            minind_lr1v = i;
            break;
        end
    end
    qhat2       = qs(qn+1-minind_lr1v);
else
    qhat1       = qs(1);
    qhat2       = qs(qn);
end

gammacf1        = qhat1';
gammacf2        = qhat2';


end
