%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the p-values of the test for the
% common threshold 
% Input: 
% idxdata = data structure including dependent, independent and threshold variable
% G = the number of groups
% h = heteroskedasticity indicator
% alpha = confidence level
%
% Output:
% pval = p-value of the test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [pval]    = test_common_threshold(idxdata,G,h,alpha)

N               = idxdata.N(end); 
T               = size(idxdata,1)/N;
k               = size(idxdata.X,2);
Y               = idxdata.y;
X               = idxdata.X;
q               = idxdata.q;
n               = length(X);
NT              = length(Y);

%% threshold estimation
% qs              = quantile(q,[0.01:0.001:0.99]);
qs              = sort(unique(q));
censor          = 0.2;                                                     % trim extreme threshold
qn_start        = round(censor*length(qs));
qn_end          = round((1-censor)*length(qs));
qs              = qs(qn_start:qn_end);
qn              = length(qs);
Q_best          = realmax;  
sn              = zeros(qn,1);
for rr = 1:qn
    gamma       = qs(rr);
    [est_theta1, est_theta2, est_gi, ~, res] = est_pstr_known_threshold(idxdata,G,gamma,h); % group estimation for each given gamma
    sn(rr)      = res'*res;
    Q           = sn(rr);
     if Q < Q_best
         Q_best    = Q;

         hattheta1 = est_theta1;
         hattheta2 = est_theta2;
         gi        = est_gi;
         qhat      = gamma;
         res_best  = res;
     end
end
smin            = Q_best;
sighat          = smin/n;
gamma_best      = qhat;

%% split-sample estimation for different threshold
resQ            = res_best'*res_best;
[~,~,~,~,~,gi_best,~,~,unresQ] = est_pstr_grid(idxdata,G,h,alpha,qs);
lrtest          = (resQ-unresQ)/unresQ*NT;
eall            = [];    
for g = 1:G                            
    NN          = 1:N;            
    gi          = logical(gi_best);            
    this_group  = gi(:,g);                    
    g_index     = NN(this_group);                    
    g_data      = idxdata(ismember(idxdata.N,g_index),:);                    
    Ng          = sum(gi(:,g));                                         
    gX          = g_data.X;                
    gy          = g_data.y;                
    gq          = g_data.q;                                        
    i1          = (gq<=gamma);             
    i2          = ~(i1);                                             
    x1          = gX(i1,:);            
    x2          = gX(i2,:);                         
    y1          = gy(i1,:);                    
    y2          = gy(i2,:);            
    mi1         = inv(x1'*x1);            
    mi2         = inv(x2'*x2);        
    
    beta1       = mi1*(x1'*y1);            
    beta2       = mi2*(x2'*y2);                 
    e1          = y1-x1*beta1;            
    e2          = y2-x2*beta2;           
    eful        = [e1;e2];             
    sigma2_g(g) = eful'*eful/(Ng*T);    
    eall        = [eall;eful];          
    
    
    nonpar_     = 1;                
    r2          = (gX*(beta1-beta2)).*(gX*(beta1-beta2));               
    r1          = r2.*((eful.*eful)*ones(1,length(r2(1,:))))/sigma2_g(g);           
    gqx         = [gq.^0,gq.^1,gq.^2];                
    gqh         = [qhat.^0,qhat.^1,qhat.^2];                
    m1          = (r2'/gqx')';                
    m2          = (r1'/gqx')';            
    
    g2          = gqh*m1;                
    g1          = gqh*m2;            
    if nonpar_== 2            
        sigq    = sqrt(mean((gq-mean(gq)').^2));                    
        hband   = 2.344*sigq/(Ng^(.2));                    
        u       = (qhat-gq)/hband;                   
        u2      = u.^2;                    
        f       = mean((1-u2).*(u2<=1))'*(1.5/(hband^2));                       
        df      = -mean((1-u2).*(u2<=1))'*(1.5/(hband^2));                    
        eps     = r2-gqx*m1;                    
        sige    = (eps'*eps)/(Ng-3);
                
        hband   = sige/(4*f*((m1(3)+(m1(2)+2*m1(3)*qhat)*df/f)^2));                    
        u2      = ((qhat-gq)/hband).^2;                    
        kh      = ((1-u2)*.75/hband).*(u2<=1);                    
        g2      = mean(kh.*(r2*ones(1,length(kh(1,:)))))';                    
        g1      = mean(kh.*(r1*ones(1,length(kh(1,:)))))';                          
    end    
    if g == 1    
        g21 = g2;                                    
    end  
    eta2_g(g)   = g1/g2;    
    mu_g1(g)    = g2./g21;            
    clear e1 e2 eful x1 x2 y1 y2 i1 i2 r1 r2 eps            
end
sigma2              = eall'*eall/(N*T);
for g = 1:G    
    tilde_eta2(g)   = sigma2_g(g)/sigma2*eta2_g(g);            
    rho_g(g)        = mu_g1(g)*Ng/N/tilde_eta2(g);            
end

rho_g       = rho_g/sum(rho_g);
L           = 20/max(min(rho_g),0.05);
[asydist]   = brownian_motion_simul(500,L,rho_g,tilde_eta2,400);
pval        = 1-sum(lrtest>sort(asydist))/length(asydist);

clear asydist

end